﻿-- Arena Style for compact raid frame more buffs and debuffs, and show debuffs outside of the unit frame


-- Initialized database defaults
local defaults = {enable = true,buffnum = 6,debuffnum = 10,min = 5,cc = true}

ArenaStyleDB = ArenaStyleDB or defaults
ArenaStyle = CreateFrame("Frame")
ArenaStyle.cache = {}

-- default buff number
local DEFAULT_BUFF = 3

-- default debuff number
local DEFAULT_DEBUFF = 3

local function log(msg) DEFAULT_CHAT_FRAME:AddMessage("|cffCD32CDArenaStyle:|r " .. msg) end

function ArenaStyle:OnLoad()
	-- Initialize the db
	self.db = ArenaStyleDB
	for k,v in pairs(defaults) do 
		if self.db[k] == nil then self.db[k] = defaults[k] end
	end
	-- Initialize command table
	SlashCmdList["ArenaStyle"] = function(msg) self:Command(msg, tbl) end
	SLASH_ArenaStyle1 = "/as"
	SLASH_ArenaStyle2 = "/ArenaStyle"
	log("type /as to config ArenaStyle.")
	-- Register event
	--self:RegisterEvent("RAID_ROSTER_UPDATE")
	--self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	--self:RegisterEvent("UNIT_PET")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("UNIT_AURA")
	CompactRaidFrameContainer:HookScript("OnEvent",ArenaStyle.OnRosterUpdate)
	CompactRaidFrameContainer:HookScript("OnHide",ArenaStyle.ResetStyle)
	CompactRaidFrameContainer:HookScript("OnShow",ArenaStyle.OnRosterUpdate)
end

-- Command table
function ArenaStyle:Command(msg , tbl)
	local cmdtbl = tbl or 
	{
		["buff"] = function(arg)
			if tonumber(arg) and tonumber(arg) >= DEFAULT_BUFF then 
				self.db.buffnum = tonumber(arg) 
				log("buff number set to be "..self.db.buffnum) 
				self:ResetStyle() 
				self:OnRosterUpdate()
			else 
				log("invalid input(must be numbers and over 3)") 
			end
		end,
		["debuff"] = function(arg) 
			if tonumber(arg) and tonumber(arg) >= DEFAULT_DEBUFF then 
				self.db.debuffnum = tonumber(arg) 
				log("debuff number set to be "..self.db.debuffnum)
				self:ResetStyle() 
				self:OnRosterUpdate()
			else
				log("invalid input(must be numbers and over 3)") 
			end
		end,
		["min"] = function(arg) 
			if tonumber(arg) and tonumber(arg) >= 0 then 
				self.db.min = tonumber(arg) 
				log("the minimum number to apply arena style set to be "..self.db.min) 
				self:OnRosterUpdate() 
			else 
				log("invalid input(must be numbers and non-negative)") 
			end 
		end,
		["off"] = function() 
			self.db.enable = false
			log("set to be disabled") 
			self:ResetStyle()
		end,
		["on"] = function() 
			self.db.enable = true
			log("set to be enabled") 
			self:OnRosterUpdate()
		end,
		["cc"] = function()
			self.db.cc = not self.db.cc
			if self.db.cc then
				log("display cc first")
			else
				log("doesn't display cc first")
			end
		end,
		--["apply"] = function() self:ApplyStyle() log("arena style applied") end,
		["status"] = function() log ("addon[".. (self.db.enable and "on]" or "off]").."  buff["..self.db.buffnum.."]".."  debuff["..self.db.debuffnum.."]".."  cc["..(self.db.enable and "on]" or "off]").."  min["..self.db.min.."]") end,
		--["reset"] = function() self:ResetStyle() log("reset to default style") end,
		--["reset"] = function() ArenaStyleDB = {}; log("reset ArenaStyle's database") end,
 		["help"] = function() log("\n'/as debuff [number]' to set debuff number\n'/as buff [number]' to set buff number\n'/as cc' to display cc first\n'/as min [number]' to set the minimum number of party or raid members to auto apply arena style\n'/as [on/off]' to enable or disable the addon") end,
		-- \n'/as status' to see the status\n'/as reset' to reset database(works after reloading)
	}
	local cmd, arg = string.split(" ", msg, 2)
	local entry = cmdtbl[cmd:lower()]
	local which = type(entry)
	if which == "function" then
		entry(arg)
	elseif which == "table" then
		self:Command(arg or "" , entry)
	else
		self:Command("help")
	end
end
--[[function CompactRaidFrameContainer_OnEvent(self, event, ...)
    if ( event == "RAID_ROSTER_UPDATE" or event == "PARTY_MEMBERS_CHANGED" ) then
        CompactRaidFrameContainer_UpdateDisplayedUnits(self);
        CompactRaidFrameContainer_TryUpdate(self);
    elseif ( event == "UNIT_PET" ) then
        if ( self.displayPets ) then
            local unit = ...;
            if ( unit == "player" or strsub(unit, 1, 4) == "raid" or strsub(unit, 1, 5) == "party" ) then
                CompactRaidFrameContainer_TryUpdate(self);
            end
        end
    end
end]]
-- When roster updated, auto apply arena style or reset style
function ArenaStyle:OnRosterUpdate()
	local _,areaType = IsInInstance()
	if areaType == "raid" then ArenaStyle:ResetStyle() return end
	if not ArenaStyle.db.enable then return end
	if not CompactRaidFrameContainer:IsVisible() then return end
	local n = max (GetNumRaidMembers(), GetNumPartyMembers())
	if n <= ArenaStyle.db.min and n > 0 then ArenaStyle:ApplyStyle() else ArenaStyle:ResetStyle() end
end

-- If in raid reset style
function ArenaStyle:OnZoneChanged()
	local _,areaType = IsInInstance()
	self:ResetStyle()
	if areaType ~= "raid" then self:ApplyStyle() end
end

-- Apply arena style to the compact raid frame
function ArenaStyle:ApplyStyle()
	for i = 1,80 do 
		local f = _G["CompactRaidFrame"..i]
		if f and not self.cache[f] and f.inUse and f.maxBuffs ~= 0 and #f.buffFrames == DEFAULT_BUFF and f.unit and not strfind(f.unit,"pet") and not strfind(f.unit,"target") then
			self:ApplyFrame(f)
			self:UpdateAura(f.unit)
		end
		if f and not f.inUse and self.cache[f] then
			self:ResetFrame(f)
		end
	end
end

-- data from LoseControl
local spellIds = {
	-- Death Knight
	[47481] = "CC",		-- Gnaw (Ghoul)
	[49203] = "CC",		-- Hungering Cold
	[91797] = "CC",		-- Monstrous Blow (Super ghoul)
	[47476] = "Silence",	-- Strangulate
	--[45524] = "Snare",	-- Chains of Ice (can also be a root with Chilblains but no way to differentiate?)
	--[55666] = "Snare",	-- Desecration
	--[50040] = "Snare",	-- Chilblains
	-- Druid
	[5211]  = "CC",		-- Bash (also Shaman Spirit Wolf ability)
	[33786] = "CC",		-- Cyclone
	[2637]  = "CC",		-- Hibernate
	[22570] = "CC",		-- Maim
	[9005]  = "CC",		-- Pounce
	[81261] = "Silence",	-- Solar Beam
	[339]   = "Root",	-- Entangling Roots
	[45334] = "Root",	-- Feral Charge Effect
	--[58179] = "Snare",	-- Infected Wounds
	--[61391] = "Snare",	-- Typhoon
	-- Hunter
	[3355]  = "CC",		-- Freezing Trap Effect
	[24394] = "CC",		-- Intimidation
	[1513]  = "CC",		-- Scare Beast
	[19503] = "CC",		-- Scatter Shot
	[19386] = "CC",		-- Wyvern Sting
	[34490] = "Silence",	-- Silencing Shot
	[19306] = "Root",	-- Counterattack
	[19185] = "Root",	-- Entrapment
	--[35101] = "Snare",	-- Concussive Barrage
	--[5116]  = "Snare",	-- Concussive Shot
	--[13810] = "Snare",	-- Frost Trap Aura
	--[61394] = "Snare",	-- Glyph of Freezing Trap
	--[2974]  = "Snare",	-- Wing Clip
	-- Hunter Pets
	[50519] = "CC",		-- Sonic Blast (Bat)
	[50541] = "Disarm",	-- Snatch (Bird of Prey)
	--[54644] = "Snare",	-- Froststorm Breath (Chimera)
	[50245] = "Root",	-- Pin (Crab)
	--[50271] = "Snare",	-- Tendon Rip (Hyena)
	[90337] = "CC",		-- Bad Manner (Monkey)
	[54706] = "Root",	-- Venom Web Spray (Silithid)
	[4167]  = "Root",	-- Web (Spider)
	-- Mage
	[44572] = "CC",		-- Deep Freeze
	[31661] = "CC",		-- Dragon's Breath
	[12355] = "CC",		-- Impact
	[83047] = "CC",		-- Improved Polymorph
	[118]   = "CC",		-- Polymorph
	[82691] = "CC",		-- Ring of Frost
	[18469] = "Silence",	-- Silenced - Improved Counterspell
	[64346] = "Disarm",	-- Fiery Payback
	[33395] = "Root",	-- Freeze (Water Elemental)
	[122]   = "Root",	-- Frost Nova
	[83302] = "Root",	-- Improved Cone of Cold
	[55080] = "Root",	-- Shattered Barrier
	--[11113] = "Snare",	-- Blast Wave
	--[6136]  = "Snare",	-- Chilled (generic effect, used by lots of spells [looks weird on Improved Blizzard, might want to comment out])
	--[120]   = "Snare",	-- Cone of Cold
	--[116]   = "Snare",	-- Frostbolt
	--[44614] = "Snare",	-- Frostfire Bolt
	--[31589] = "Snare",	-- Slow
	-- Paladin
	[853]   = "CC",		-- Hammer of Justice
	[2812]  = "CC",		-- Holy Wrath
	[20066] = "CC",		-- Repentance
	[10326] = "CC",		-- Turn Evil
	[31935] = "Silence",	-- Avenger's Shield
	--[63529] = "Snare",	-- Dazed - Avenger's Shield
	--[20170] = "Snare",	-- Seal of Justice (100% movement snare; druids and shamans might want this though)
	-- Priest
	[605]   = "CC",		-- Mind Control
	[64044] = "CC",		-- Psychic Horror
	[8122]  = "CC",		-- Psychic Scream
	[9484]  = "CC",		-- Shackle Undead
	[87204] = "CC",		-- Sin and Punishment
	[15487] = "Silence",	-- Silence
	--[64058] = "Disarm",	-- Psychic Horror (duplicate debuff names not allowed atm, need to figure out how to support this later)
	[87194] = "Root",	-- Paralysis
	--[15407] = "Snare",	-- Mind Flay
	-- Rogue
	[2094]  = "CC",		-- Blind
	[1833]  = "CC",		-- Cheap Shot
	[1776]  = "CC",		-- Gouge
	[408]   = "CC",		-- Kidney Shot
	[6770]  = "CC",		-- Sap
	[76577] = "CC",		-- Smoke Bomb
	[1330]  = "Silence",	-- Garrote - Silence
	[18425] = "Silence",	-- Silenced - Improved Kick
	[51722] = "Disarm",	-- Dismantle
	--[31125] = "Snare",	-- Blade Twisting
	--[3409]  = "Snare",	-- Crippling Poison
	--[26679] = "Snare",	-- Deadly Throw
	-- Shaman
	[76780] = "CC",		-- Bind Elemental
	[61882] = "CC",		-- Earthquake
	[51514] = "CC",		-- Hex
	[39796] = "CC",		-- Stoneclaw Stun
	[64695] = "Root",	-- Earthgrab (Earth's Grasp)
	[63685] = "Root",	-- Freeze (Frozen Power)
	[3600]  = "Snare",	-- Earthbind (5 second duration per pulse, but will keep re-applying the debuff as long as you stand within the pulse radius)
	--[8056]  = "Snare",	-- Frost Shock
	--[8034]  = "Snare",	-- Frostbrand Attack
	-- Warlock
	[93986] = "CC",		-- Aura of Foreboding
	[89766] = "CC",		-- Axe Toss (Felguard)
	[710]   = "CC",		-- Banish
	[6789]  = "CC",		-- Death Coil
	[54786] = "CC",		-- Demon Leap
	[5782]  = "CC",		-- Fear
	[5484]  = "CC",		-- Howl of Terror
	[6358]  = "CC",		-- Seduction (Succubus)
	[30283] = "CC",		-- Shadowfury
	[24259] = "Silence",	-- Spell Lock (Felhunter)
	[31117] = "Silence",	-- Unstable Affliction
	--[93974] = "Root",	-- Aura of Foreboding (duplicate debuff names not allowed atm, need to figure out how to support this later)
	--[18118] = "Snare",	-- Aftermath
	--[18223] = "Snare",	-- Curse of Exhaustion
	--[63311] = "Snare",	-- Shadowsnare (Glyph of Shadowflame)
	-- Warrior
	[7922]  = "CC",		-- Charge Stun
	[12809] = "CC",		-- Concussion Blow
	[6544]  = "CC",		-- Heroic Leap
	[20253] = "CC",		-- Intercept
	[5246]  = "CC",		-- Intimidating Shout
	[46968] = "CC",		-- Shockwave
	[85388] = "CC",		-- Throwdown
	[18498] = "Silence",	-- Silenced - Gag Order
	[676]   = "Disarm",	-- Disarm
	[23694] = "Root",	-- Improved Hamstring
	--[1715]  = "Snare",	-- Hamstring
	--[12323] = "Snare",	-- Piercing Howl
	-- Other
	[30217] = "CC",		-- Adamantite Grenade
	[67769] = "CC",		-- Cobalt Frag Bomb
	[30216] = "CC",		-- Fel Iron Bomb
	[13327] = "CC",		-- Reckless Charge
	[56]    = "CC",		-- Stun
	[20549] = "CC",		-- War Stomp
	[25046] = "Silence",	-- Arcane Torrent
	[39965] = "Root",	-- Frost Grenade
	[55536] = "Root",	-- Frostweave Net
	[13099] = "Root",	-- Net-o-Matic
	--[29703] = "Snare",	-- Dazed
	-- CC Immunities
	[19574] = "Immune",	-- Bestial Wrath (Hunter pet)
	[46924] = "Immune",	-- Bladestorm (Warrior)
	[19263] = "Immune",	-- Deterrence [except aoe?] (Hunter)
	[47585] = "Immune",	-- Dispersion (Priest)
	[642]   = "Immune",	-- Divine Shield (Paladin)
	[45438] = "Immune",	-- Ice Block (Mage)
	--[54216] = "Immune",	-- Master's Call (Hunter pet + target, root and snare immune only)
	-- Spell Immunities
	[48707] = "ImmuneSpell",	-- Anti-magic Shell (Death Knight)
	[31224] = "ImmuneSpell",	-- Cloak of Shadows (Rogue)
	[8178]  = "ImmuneSpell",	-- Grounding Totem Effect [exept aoe] (Shaman)
	[23920] = "ImmuneSpell",	-- Spell Reflection [except aoe] (Warrior)
	-- Other Immunities - optional, not sure how to handle these; many have durations that are quite long
	--[18499] = "ImmuneFearIncapacitate",	-- Berserker Rage (Warrior)
	--[45182] = "ImmuneDamage",	-- Cheating Death [only 90% immune] (Rogue)
	--[5277]  = "ImmuneMelee",	-- Evasion [only +50% immune to melee, +25% to ranged] (Rogue)
	--[48792] = "ImmuneStun",	-- Icebound Fortitude [stuns] (Death Knight)
	--[49039] = "ImmuneFearPoly",	-- Lichborne [charm, fear, sleep, hex, polymorph] (Death Knight)
	--[51271] = "ImmuneMove",	-- Pillar of Frost (Death Knight)
}
local abilities = {} -- localized names are saved here
for k, v in pairs(spellIds) do
	local name = GetSpellInfo(k)
	if name then
		abilities[name] = v
	else
		log(" unknown spellId: " .. k)
	end
end

local function isPriorityDebuff(unit, index, filter)
    local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff = UnitDebuff(unit, index, filter);
    if ( spellId == 30108 or spellId == 34914 ) then  
		return true
    elseif ( abilities[name] ) then
		return true
	end
	return false
end
-- Update aura for each unit
function ArenaStyle:UpdateAura(uid)
	for f,v in pairs(self.cache) do
		if f.unit == uid then
			local filter = nil
			local buffNum = 1
			local index = 1
			while buffNum <= self.db.buffnum do
				local buffName = UnitBuff(uid, index, filter);
				if ( buffName ) then
					if ( CompactUnitFrame_UtilShouldDisplayBuff(uid, index, filter) ) then
						if buffNum > DEFAULT_BUFF then
							local buffFrame = v.buffFrames[buffNum]
							CompactUnitFrame_UtilSetBuff(buffFrame, uid, index, filter)
						end
						buffNum = buffNum + 1
					end
				else
					break
				end
				index = index + 1
			end
			for i=buffNum, self.db.buffnum do
				local buffFrame = v.buffFrames[i]
				if buffFrame then
					buffFrame:Hide()
				end
			end
			local debuffNum = 1
			index = 1
			if ( f.optionTable.displayOnlyDispellableDebuffs ) then
				filter = "RAID"
			end
			if self.db.cc then
				while debuffNum <= self.db.debuffnum do
					local debuffName = UnitDebuff(uid, index, nil)
					if ( debuffName ) then
						if ( CompactUnitFrame_UtilShouldDisplayDebuff(uid, index, nil) and isPriorityDebuff(uid, index, nil)) then
							local debuffFrame = v.debuffFrames[debuffNum]
							debuffFrame:SetSize(f.buffFrames[3]:GetSize()*1.5,f.buffFrames[3]:GetSize()*1.5)
							CompactUnitFrame_UtilSetDebuff(debuffFrame, uid, index, nil)
							debuffNum = debuffNum + 1
						end
					else
						break
					end
					index = index + 1
				end
				index = 1
				while debuffNum <= self.db.debuffnum do
					local debuffName = UnitDebuff(uid, index, filter)
					if ( debuffName ) then
						if ( CompactUnitFrame_UtilShouldDisplayDebuff(uid, index, filter) and not isPriorityDebuff(uid, index, filter)) then
							local debuffFrame = v.debuffFrames[debuffNum]
							debuffFrame:SetSize(f.buffFrames[3]:GetSize())
							CompactUnitFrame_UtilSetDebuff(debuffFrame, uid, index, filter)
							debuffNum = debuffNum + 1
						end
					else
						break
					end
					index = index + 1
				end
			else
				while debuffNum <= self.db.debuffnum do
					local debuffName = UnitDebuff(uid, index, filter)
					if ( debuffName ) then
						if ( CompactUnitFrame_UtilShouldDisplayDebuff(uid, index, filter) ) then
							local debuffFrame = v.debuffFrames[debuffNum]
							debuffFrame:SetSize(f.buffFrames[3]:GetSize())
							CompactUnitFrame_UtilSetDebuff(debuffFrame, uid, index, filter)
							debuffNum = debuffNum + 1
						end
					else
						break
					end
					index = index + 1
				end
			end
			for i=debuffNum, self.db.debuffnum do
				local debuffFrame = v.debuffFrames[i];
				if debuffFrame then
					debuffFrame:Hide()
				end
			end
			break
		end
	end
end
-- Apply style for each frame
function ArenaStyle:ApplyFrame(f)
	self.cache[f] = {}
	f:SetScript("OnSizeChanged",function() ArenaStyle:ResetFrame(f) ArenaStyle:ApplyFrame(f) end) 
	if not self.cache[f].buffFrames then self.cache[f].buffFrames = {} end
	if not self.cache[f].debuffFrames then self.cache[f].debuffFrames = {} end
	for j = DEFAULT_BUFF + 1, self.db.buffnum do
		if not self.cache[f].buffFrames[j] then
			self.cache[f].buffFrames[j] = CreateFrame("Button",nil,UIParent,"ArenaStyleBuffTemplate")
			self.cache[f].buffFrames[j].unit = f.unit
			--self.cache[f].buffFrames[j]:EnableMouse(false)
			if j == DEFAULT_BUFF + 1 then
				self.cache[f].buffFrames[j]:SetPoint("RIGHT",f.buffFrames[3],"LEFT")
			else
				self.cache[f].buffFrames[j]:SetPoint("RIGHT",self.cache[f].buffFrames[j-1],"LEFT")
			end
			self.cache[f].buffFrames[j]:SetSize(f.buffFrames[3]:GetSize())
		end
	end
	for j = 1,self.db.debuffnum do
		if not self.cache[f].debuffFrames[j] then
			self.cache[f].debuffFrames[j] = CreateFrame("Button",nil,UIParent,"ArenaStyleDebuffTemplate")
			self.cache[f].debuffFrames[j].unit = f.unit
			--self.cache[f].debuffFrames[j]:EnableMouse(false)
			if j == 1 then
				self.cache[f].debuffFrames[j]:ClearAllPoints()
				self.cache[f].debuffFrames[j]:SetPoint("BOTTOMLEFT", f, "BOTTOMRIGHT",1,10) 
			else
				self.cache[f].debuffFrames[j]:SetPoint("BOTTOMLEFT",self.cache[f].debuffFrames[j-1],"BOTTOMRIGHT")
			end
			--f.debuffFrames[j]:SetSize(f.buffFrames[3]:GetSize())
			self.cache[f].debuffFrames[j]:SetSize(f.buffFrames[3]:GetSize())
			self.cache[f].debuffFrames[j]:Hide()
		end
	end
	for j = 1,#f.debuffFrames do
		f.debuffFrames[j]:Hide()
		f.debuffFrames[j]:SetScript("OnShow",f.debuffFrames[j].Hide) 
	end
end
-- Reset to the original style
function ArenaStyle:ResetStyle()
	for f,_ in pairs(ArenaStyle.cache) do
		ArenaStyle:ResetFrame(f)
	end
end
-- Reset style to each cached frame
function ArenaStyle:ResetFrame(f)
	for k,v in pairs(self.cache[f].buffFrames) do
		if v then
			v:Hide()
		end
	end
	for k,v in pairs(self.cache[f].debuffFrames) do
		if v then
			v:Hide()
		end
	end
	f:SetScript("OnSizeChanged",nil)
	for j = 1,#f.debuffFrames do
		f.debuffFrames[j]:SetScript("OnShow",nil) 
	end
	self.cache[f] = nil
end
function ArenaStyle:OnCombat()
	-- todo
end
function ArenaStyle:LeaveCombat()
	-- todo
end

-- Event handling
local function OnEvent(self,event,...)
	if event == "VARIABLES_LOADED" then self:OnLoad() end
	if event == "RAID_ROSTER_UPDATE" or event == "PARTY_MEMBERS_CHANGED" or event == "UNIT_PET" then self:OnRosterUpdate() end
	if event == "PLAYER_ENTERING_WORLD" then self:OnRosterUpdate() end
	if event == "PLAYER_REGEN_DISABLED" then self:OnCombat() end
	if event == "PLAYER_REGEN_ENABLED" then self:LeaveCombat() end
	if event == "UNIT_AURA" then self:UpdateAura(...) end
end

ArenaStyle:SetScript("OnEvent",OnEvent)
ArenaStyle:RegisterEvent("VARIABLES_LOADED")
_G.ArenaStyle = ArenaStyle